//-------------------------------------------------------------------------------------------------
// Adapted from https://github.com/TheRealMJP/Shadows - MIT License
//-------------------------------------------------------------------------------------------------

#if defined(SHADOW_MAP_PRESENT) && !defined(VERTEX_SHADER)

const vec2 shadowMapSize = vec2(SHADOW_MAP_SIZE, SHADOW_MAP_SIZE);

float pcfSample(in sampler2DArrayShadow shadowMap, in vec2 base_uv, in float u, in float v, in vec2 texelSize, in float cascade,  in float depth)
{
	vec2 uv = base_uv + vec2(u, v) * texelSize;
	return texture(shadowMap, vec4(uv, cascade, depth));
}

float sampleShadowPCF(in sampler2DArrayShadow shadowMap, in vec3 shadowPos, in float cascade)
{
	float lightDepth = shadowPos.z;

	vec2 texelSize = 1.0 / shadowMapSize;
	vec2 uv = shadowPos.xy * shadowMapSize;

	vec2 base_uv;
	base_uv.x = floor(uv.x + 0.5);
	base_uv.y = floor(uv.y + 0.5);

	float s = (uv.x + 0.5 - base_uv.x);
	float t = (uv.y + 0.5 - base_uv.y);

	base_uv -= vec2(0.5, 0.5);
	base_uv *= texelSize;

	float sum = 0;

	float uw0 = (3 - 2 * s);
	float uw1 = (1 + 2 * s);

	float u0 = (2 - s) / uw0 - 1;
	float u1 = s / uw1 + 1;

	float vw0 = (3 - 2 * t);
	float vw1 = (1 + 2 * t);

	float v0 = (2 - t) / vw0 - 1;
	float v1 = t / vw1 + 1;

	sum += uw0 * vw0 * pcfSample(shadowMap, base_uv, u0, v0, texelSize, cascade, lightDepth);
	sum += uw1 * vw0 * pcfSample(shadowMap, base_uv, u1, v0, texelSize, cascade, lightDepth);
	sum += uw0 * vw1 * pcfSample(shadowMap, base_uv, u0, v1, texelSize, cascade, lightDepth);
	sum += uw1 * vw1 * pcfSample(shadowMap, base_uv, u1, v1, texelSize, cascade, lightDepth);

	return sum * 1.0 / 16;
}

#endif
