// analysis: the cheaper approximate doesn't really improve performance by much
// analysis: removing tonemap doesn't really improve performance by much
// conclusion: keep tonemap

// #define FAKE_ACES
// #define CHEAP_TONEMAP
// #define NO_TONEMAP

#ifdef FAKE_ACES

vec3 from_gamma(vec3 color) {
    return color * color;
}

vec3 tonemap(vec3 color) {
    vec3 power = pow(color, vec3(1.6));
    return sqrt(clamp(power / (power * 0.993014012791 + 0.172903609999), 0.0, 1.0));
}

vec3 reverse_tonemap(vec3 color) {
    // include gamma
    return pow((0.172903609999 * color) / (1.0 - 0.993014012791 * color), vec3(2.0 / 1.6));
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(6.0 * max(0.0, color_average - 0.8) + emission, 0.0, 1.0);
}

#elif defined(NO_TONEMAP)

vec3 tonemap(vec3 color) {
    return clamp(color, 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return color;
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(color_average + emission, 0.0, 1.0);
}

#elif defined(CHEAP_TONEMAP)

vec3 tonemap(vec3 color) {
    return clamp((color - vec3(0.4)) / (color + vec3(1.0)) + vec3(0.4), 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return -color / (color - vec3(0.4 + 1.0));
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(2.7 * max(0.0, color_average - 0.63) + emission, 0.0, 1.0);
}

#else

vec3 tonemap(vec3 color) {
    return clamp(-exp(vec3(0.69314717) - 1.386294 * color) * 0.5 + 1, 0.0, 1.0);
}

vec3 reverse_tonemap(vec3 color) {
    return (log(-2.0 * (color - vec3(1.0))) - vec3(0.69314717)) / vec3(-1.386294);
}

float getBloom(vec3 color, float emission) {
    float color_average = (color.r + color.g + color.b) * 0.333333;
    return clamp(3.7 * max(0.0, color_average - 0.73) + emission, 0.0, 1.0);
}

#endif