#include frex:shaders/api/fragment.glsl
#include frex:shaders/api/world.glsl
#include ecos:shaders/pbr/material.glsl

void frx_materialFragment() {
  #ifdef ECOS_WATER
    vec2 obj_coord = frx_vertex.xz + frx_cameraPos.xz;
    vec2 noise_coord1 = floor(obj_coord * 8.0) / 8.0;
    vec2 noise_coord2 = noise_coord1;
    vec2 blob_coord = obj_coord;
    
    noise_coord2.y += 0.5; // make them out of phase vertically, prevent burst flash

    noise_coord1.x += frx_renderSeconds;
    noise_coord2.x -= frx_renderSeconds + 0.5;
    blob_coord.x += frx_renderSeconds * 0.5;

    noise_coord1.y *= 4.0;
    noise_coord2.y *= 4.0;

    const float scale = 32.0;
    const float scale_blob = 160.0;

    vec3 reflection = reflect(normalize(floor(frx_vertex.xyz * 8.0) / 8.0), frx_vertexNormal);
    float gloss = pow(clamp(dot(frx_skyLightVector, reflection), 0.0, 1.0), 30.0) * frx_skyLightTransitionFactor;

    float peaks = texture(noise_texture, noise_coord1 / scale).r;
    peaks += texture(noise_texture, noise_coord2 / scale).r;
    float blob = texture(noise_texture, blob_coord / scale_blob).r;
    blob = smoothstep(0.4, 1.0, blob);
    blob = max(blob, gloss);

    peaks = smoothstep(1.3 - gloss * 0.2, 2.0, peaks) * blob;
    peaks *= smoothstep(0.9, 1.0, frx_vertexNormal.y);
    // peaks = peaks * 0.8 + gloss * 0.2;
    peaks *= 0.5 + 0.5 * (1.0 - float(frx_worldIsMoonlit)) * float(frx_worldHasSkylight) * frx_skyLightTransitionFactor;

    frx_fragColor.a = mix(frx_fragColor.a * 0.6, 1.0, smoothstep(0.0, 128.0, frx_distance));
    frx_fragColor.rgb = frx_fragColor.rgb * 0.4 + 0.4 * frx_vertexColor.rgb * frx_vertexColor.rgb + vec3(0.0, 0.2, 0.0);
    frx_fragColor += peaks * vec4(1.0, 2.0, 2.0, 1.0 - frx_fragColor.a);
    frx_fragEmissive += peaks * 5.0;
  #endif
}